/*******************************************************************************
 * Copyright (c) 2013 Olivier Langella <Olivier.Langella@moulon.inra.fr>
 *
 * This file is part of XTandemPipeline.
 *
 *     XTandemPipeline is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     XTandemPipeline is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with XTandemPipeline.  If not, see <http://www.gnu.org/licenses/>.
 *
 ******************************************************************************/
package fr.inra.pappso.xtandempipeline.class_msms;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.util.HashSet;

import org.apache.log4j.Logger;

import fr.inra.pappso.xtandempipeline.MsException.MSMSException;
import fr.inra.pappso.xtandempipeline.input.fasta.FastaInputStreamParser;
import fr.inra.pappso.xtandempipeline.input.fasta.FastaReader;
import fr.inra.pappso.xtandempipeline.input.fasta.FastaReaderAccessionSet;
import fr.inra.pappso.xtandempipeline.input.fasta.FastaReaderProteinList;

public class FastaFile implements Comparable<FastaFile> {
	private static final Logger logger = Logger.getLogger(FastaFile.class);

	private String key;
	private File fastaFile;

	@Override
	public int hashCode() {
		return fastaFile.hashCode();
	}

	@Override
	public boolean equals(Object toCompareIn) {
		FastaFile toCompare = (FastaFile) toCompareIn;
		return (this.fastaFile.equals(toCompare.fastaFile));
	}

	@Override
	public int compareTo(FastaFile fileIn) {
		return (this.fastaFile.compareTo(fileIn.fastaFile));
	}

	public FastaFile(String filePath) {
		this.key = filePath;
		this.fastaFile = new File(filePath);
	}

	public String getKey() {
		return key;
	}

	public String getName() {
		return fastaFile.getName();
	}

	public boolean exists() {
		return fastaFile.exists();
	}

	public String getAbsolutePath() {
		return fastaFile.getAbsolutePath();
	}

	public void checkReadable() throws MSMSException {
		if (!this.fastaFile.exists()) {
			String message = "Fasta file \"" + this.fastaFile.getAbsolutePath()
					+ "\" does not exists";
			logger.error(message);
			throw new MSMSException(message);
		}
		if (!this.fastaFile.canRead()) {
			String message = "Unable to read Fasta file \""
					+ this.fastaFile.getAbsolutePath()
					+ "\", please check permissions";
			logger.error(message);
			throw new MSMSException(message);
		}
	}

	public InputStream getInputStream() throws MSMSException {
		checkReadable();
		try {
			return new FileInputStream(this.fastaFile);
		} catch (FileNotFoundException e) {
			String message = "Fasta file not found\""
					+ this.fastaFile.getAbsolutePath() + "\n" + e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		}
	}

	public void setProteinSequencesAndPeptidePositions(FastaFile fastaFile,
			Identification identification) throws MSMSException {
		logger.debug("setProteinSequencesAndPeptidePositions "
				+ fastaFile.getAbsolutePath());
		FastaReader transformer = new FastaReaderProteinList(identification);
		parseFastaFile(transformer);
	}

	public HashSet<String> getAccessionSet() throws MSMSException {
		logger.debug("getAccessionSet " + fastaFile.getAbsolutePath());
		FastaReaderAccessionSet transformer = new FastaReaderAccessionSet();
		parseFastaFile(transformer);
		return (transformer.getAccessionSet());

	}

	private void parseFastaFile(FastaReader transformer) throws MSMSException {
		logger.debug("getAccessionSet " + fastaFile.getAbsolutePath());
		InputStream seqIn = this.getInputStream();
		FastaInputStreamParser fastaParser = new FastaInputStreamParser(
				transformer);
		try {
			fastaParser.parseFastaInputStream(seqIn);
		} catch (Exception e) {
			String message = "Exception closing fasta file \""
					+ this.fastaFile.getAbsolutePath() + "\" \n"
					+ e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		}
		try {
			seqIn.close();
		} catch (IOException e) {
			String message = "IOException closing fasta file \""
					+ this.fastaFile.getAbsolutePath() + "\" \n"
					+ e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		}
	}

}
