/*******************************************************************************
 * Copyright (c) 2013 Olivier Langella <Olivier.Langella@moulon.inra.fr>.
 *
 * This file is part of XTandemPipeline.
 *
 *     XTandemPipeline is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     XTandemPipeline is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with XTandemPipeline.  If not, see <http://www.gnu.org/licenses/>.
 *
 ******************************************************************************/

package fr.inra.pappso.xtandempipeline.class_msms.GroupingProtein;

import java.util.HashMap;

import org.apache.log4j.Logger;

import fr.inra.pappso.xtandempipeline.MsException.MSMSException;
import fr.inra.pappso.xtandempipeline.class_msms.MsRun;
import fr.inra.pappso.xtandempipeline.class_msms.MapDb.Peptide;
import fr.inra.pappso.xtandempipeline.grouping.HashSampleScan;
import fr.inra.pappso.xtandempipeline.grouping.HashSampleScanSet;
import fr.inra.pappso.xtandempipeline.grouping.SubGroup;
import fr.inra.pappso.xtandempipeline.grouping.SubGroupSet;

/**
 * This entity regroups all peptides that share the same Mh and same sequenceLi
 *
 * this is the same concept as Peptide in GroupingProtein software
 *
 * @author langella
 *
 */
public class PeptideMass implements Comparable<PeptideMass> {
	private static final Logger logger = Logger.getLogger(PeptideMass.class);

	// modified mass
	private Double mhDiff;
	private String sequenceLi;
	private int number = 0;

	private HashSampleScanSet hashSampleScanSet = new HashSampleScanSet();
	/* List of hashSampleScans (scan number, charge) identifying this peptides */
	private HashMap<MsRun, HashMap<Integer, Integer>> msRun2scanAndCharge = new HashMap<MsRun, HashMap<Integer, Integer>>();
	private SubGroupSet subgroupSet = new SubGroupSet();

	private Peptide bestEvaluePeptide = null;

	protected PeptideMass(String sequenceLi, Double mhDiff)
			throws MSMSException {
		// logger.debug("mh=" + mh);
		this.sequenceLi = sequenceLi;
		this.mhDiff = mhDiff;
	}

	@Override
	public int hashCode() {
		String str = sequenceLi + mhDiff.toString();
		return str.hashCode();
	}

	@Override
	public boolean equals(Object toCompareIn) {
		PeptideMass toCompare = (PeptideMass) toCompareIn;
		logger.debug(this.sequenceLi + " " + this.mhDiff);
		logger.debug(toCompare.sequenceLi + " " + toCompare.mhDiff);
		return ((this.mhDiff.equals(toCompare.mhDiff)) && sequenceLi
				.equals(toCompare.sequenceLi));
	}

	@Override
	public int compareTo(PeptideMass pep) {
		if (sequenceLi.equals(pep.sequenceLi))
			return (mhDiff.compareTo(pep.mhDiff));
		else
			return (sequenceLi.compareTo(pep.sequenceLi));
	}

	protected void addPeptide(Peptide peptide) throws MSMSException {
		MsRun msRun = peptide.getMsRun();
		if (!msRun2scanAndCharge.containsKey(msRun)) {
			msRun2scanAndCharge.put(msRun, new HashMap<Integer, Integer>());
		}
		HashSampleScan sampleScan = peptide.getHashSampleScan();

		msRun2scanAndCharge.get(msRun).put(sampleScan.getScan(),
				peptide.getCharge());

		hashSampleScanSet.add(sampleScan);

		if (bestEvaluePeptide == null) {
			bestEvaluePeptide = peptide;
		}

		if (bestEvaluePeptide.get_evalue() > peptide.get_evalue()) {
			bestEvaluePeptide = peptide;
		}
	}

	public void addSubGroup(SubGroup sg) {
		this.subgroupSet.add(sg);
	}

	public SubGroupSet getSubGroupSet() {
		return this.subgroupSet;
	}

	public HashSampleScanSet getHashSampleScanSet() {
		return hashSampleScanSet;
	}

	public void setNumber(int i) {
		this.number = i;
	}

	public Integer getNumber() throws MSMSException {
		if (number == 0) {
			String message = "number==0 in PeptideMass : use the PeptideMassSet numbering function";
			logger.error(message);
			throw new MSMSException(message);
		}
		return this.number;
	}

	public String getSequenceLi() {
		return this.sequenceLi;
	}

	public Peptide getBestEvaluePeptide() {
		return this.bestEvaluePeptide;
	}

	public Double getMhDiff() {
		return this.mhDiff;
	}

	public boolean containsHashSampleScan(HashSampleScan hashSampleScan) {
		return this.hashSampleScanSet.contains(hashSampleScan);
	}

}
