/*******************************************************************************
 * Copyright (c) 2015 Olivier Langella <Olivier.Langella@moulon.inra.fr>
 *
 * This file is part of XTandemPipeline.
 *
 *     XTandemPipeline is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     XTandemPipeline is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with XTandemPipeline.  If not, see <http://www.gnu.org/licenses/>.
 *
 ******************************************************************************/

package fr.inra.pappso.xtandempipeline.sax_parse;

import org.apache.log4j.Logger;
import org.xml.sax.*;

import fr.inra.pappso.xtandempipeline.Utils;
import fr.inra.pappso.xtandempipeline.MsException.MSMSException;
import fr.inra.pappso.xtandempipeline.class_msms.*;
import fr.inra.pappso.xtandempipeline.class_msms.IdentificationDataFile.IdentificationDataSource;
import fr.inra.pappso.xtandempipeline.class_msms.MapDb.PeptideProphet;

public class HandlerPepXmlResultFile extends SaxBaseHandler {
	protected final Logger logger = Logger
			.getLogger(HandlerPepXmlResultFile.class);

	private IdentificationDataSource identificationDataSource;

	protected PeptideProphet currentPeptide;

	protected Identification identification;

	protected Match currentMatch;

	private Long currentStartScan;

	private Long currentEndScan;

	private Integer currentZ;

	private Double currentRt;

	private MzxmlFile mzXmlFile;

	private FastaFile fastaFile;

	private Double currentPeptideProphetProbability = null;

	private Double currentPeptideInterProphetProbability = null;

	private Double currentExpectedValue = null;

	private String currentSearchEngine;

	private Double currentPrecursorNeutralMass;

	public HandlerPepXmlResultFile(Identification L, IdentificationDataSource S) {
		super();
		if (L == null) {
			identification = new Identification();
		} else {
			identification = L;
		}
		identificationDataSource = S;
	}

	@Override
	public void startDocument() throws SAXException {
		try {
			identification.getMsRunSet().getInstance(identificationDataSource);
			logger.info("Parse pepxml file in normal Mode : "
					+ identificationDataSource.getName());
		} catch (Exception e) {
			String message = "ERROR reading pepxml file :\n" + e.getMessage();
			logger.error(message);
			throw new SAXException(message);
		}
	}

	@Override
	public void endDocument() throws SAXException {
	}

	// <msms_pipeline_analysis date="2015-01-19T13:28:41"
	// xmlns="http://regis-web.systemsbiology.net/pepXML"
	// xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
	// xsi:schemaLocation="http://sashimi.sourceforge.net/schema_revision/pepXML/pepXML_v117.xsd"
	// summary_xml="/gorgone/pappso/abrf_2015/mzXML/JD_06232014_sample1-A.pep.xml">
	public void startElementmsms_pipeline_analysis(String nameSpaceUri,
			Attributes attrs) {
		String originalFilename = attrs.getValue("summary_xml");
	}

	// <msms_run_summary
	// base_name="/gorgone/pappso/abrf_2015/mzXML/JD_06232014_sample1-A"
	// msManufacturer="Thermo Scientific" msModel="Q Exactive"
	// raw_data_type="raw" raw_data=".mzXML">
	public void startElementmsms_run_summary(String nameSpaceUri,
			Attributes attrs) {
		this.mzXmlFile = new MzxmlFile(attrs.getValue("base_name")
				+ attrs.getValue("raw_data"));
	}

	// <search_database
	// local_path="/gorgone/pappso/abrf_2015/fasta/iPRG2015.fasta" type="AA"/>
	public void startElementsearch_database(String nameSpaceUri,
			Attributes attrs) {
		this.fastaFile = new FastaFile(attrs.getValue("local_path"));
	}

	// <search_summary
	// base_name="/gorgone/pappso/moulon/users/Olivier/Delphine_Pecqueur/mzXML/330_01_01_C3G3_150122_01"
	// search_engine="X! Tandem (k-score)" precursor_mass_type="monoisotopic"
	// fragment_mass_type="monoisotopic" search_id="1">
	public void startElementsearch_summary(String nameSpaceUri, Attributes attrs) {
		currentSearchEngine = attrs.getValue("search_engine");
	}

	// <spectrum_query spectrum="JD_06232014_sample1-A.00005.00005.2"
	// start_scan="5" end_scan="5" precursor_neutral_mass="819.347822"
	// assumed_charge="2" index="1" retention_time_sec="1.4">
	// <spectrum_query spectrum="JD_06232014_sample1-A.00006.00006.2"
	// start_scan="6" end_scan="6" precursor_neutral_mass="870.405029296875"
	// assumed_charge="2" index="1">

	public void startElementspectrum_query(String nameSpaceUri, Attributes attrs)
			throws MSMSException {
		currentStartScan = new Long(attrs.getValue("start_scan"));
		currentEndScan = new Long(attrs.getValue("end_scan"));
		if (!currentStartScan.equals(currentEndScan)) {
			String message = "ERROR reading pepxml file :\n"
					+ "unable to read search results from '"
					+ this.currentSearchEngine + "' as start "
					+ currentStartScan + " and end " + currentEndScan
					+ " scans are different";
			logger.error(message);
			throw new MSMSException(message);
		}
		currentZ = new Integer(attrs.getValue("assumed_charge"));
		if (attrs.getValue("retention_time_sec") == null) {
			String message = "ERROR reading pepxml file :\n"
					+ "unable to read search results from '"
					+ this.currentSearchEngine
					+ "' as retention time is not given in spectrum_query elements";
			logger.warn(message);
			// throw new MSMSException(message);

			currentRt = (double) 0;
		} else {
			currentRt = new Double(attrs.getValue("retention_time_sec"));
		}
		currentPrecursorNeutralMass = new Double(
				attrs.getValue("precursor_neutral_mass"))
				+ Utils.mhplus;
	}

	// <search_hit hit_rank="5" peptide="MKDFSTK" peptide_prev_aa="K"
	// peptide_next_aa="R" protein="sp|P34218|SAS3_YEAST" num_tot_proteins="1"
	// num_matched_ions="2" tot_num_ions="12" calc_neutral_pep_mass="871.410939"
	// massdiff="-1.006264" num_tol_term="2" num_missed_cleavages="1"
	// num_matched_peptides="190">
	public void startElementsearch_hit(String nameSpaceUri, Attributes attrs)
			throws MSMSException {
		String proteinName = attrs.getValue("protein");

		Protein protein = new Protein();
		protein.set_description(proteinName);
		if (identification.contain_protein(protein)) {
			// new protein, new match
			currentMatch = new Match();
			currentMatch.set_protein_match(protein);
			identification.add_protein(protein, currentMatch);
		}

		else {
			// get existing match for this protein
			currentMatch = (Match) identification.get_match(protein);
		}
		protein = currentMatch.get_protein_match();
		protein.setDatabase(identification.getDatabaseSet().getInstance(
				this.fastaFile.getName()));

		this.currentPeptide = new PeptideProphet();
		currentPeptide.set_sample_id(
				identification.getMsRunSet().getInstance(
						identificationDataSource), identificationDataSource);

		currentPeptide.set_pre(attrs.getValue("peptide_prev_aa"));
		currentPeptide.set_post(attrs.getValue("peptide_next_aa"));
		// peptide="SDLSDER"
		currentPeptide.set_sequence(attrs.getValue("peptide"));

		currentPeptide.set_scan(this.currentEndScan.intValue());
		Double calc_neutral_pep_mass = new Double(
				attrs.getValue("calc_neutral_pep_mass"))
				+ Utils.mhplus;
		Double massdiff = new Double(attrs.getValue("massdiff"));

		currentPeptide.set_mhplus_theo(calc_neutral_pep_mass);
		currentPeptide.set_deltamass(massdiff);
		currentPeptide.set_mhplus_obser(currentPrecursorNeutralMass);
		currentPeptide.set_charge(this.currentZ);
		currentPeptide.setRt(this.currentRt.floatValue());

		/*
		 * if (Math.abs(massdiff) > 1) { String message =
		 * "massdiff is too high"; logger.error(message); throw new
		 * MSMSException(message); }
		 */
		// valeur specifique de la séquence matche
		// peptide.set_start(Integer.parseInt(attrs.getValue("start")));
		// peptide.set_stop(Integer.parseInt(attrs.getValue("end")));
		// peptide.set_evalue(Float.valueOf(attrs.getValue("expect")));
		// peptide.set_hypercorr(Float.valueOf(attrs.getValue("hyperscore")));

	}

	public void endElementsearch_hit(String nameSpaceUri, String data) {
		if (currentPeptideProphetProbability != null) {
			currentPeptide
					.setProphetProbability(currentPeptideProphetProbability);
		}
		if (currentPeptideInterProphetProbability != null) {
			currentPeptide
					.setInterProphetProbability(currentPeptideInterProphetProbability);
		}
		currentPeptide.setSearchEngine(this.currentSearchEngine);

		identification.add_peptide(this.currentPeptide);
		currentMatch.add_peptide_match(currentPeptide);

		currentPeptideProphetProbability = null;
		currentPeptideInterProphetProbability = null;
		currentExpectedValue = null;

	}

	// <peptideprophet_result probability="0.0245"
	// all_ntt_prob="(0.0000,0.0093,0.0245)">
	public void startElementpeptideprophet_result(String nameSpaceUri,
			Attributes attrs) throws MSMSException {
		this.currentPeptideProphetProbability = Double.parseDouble(attrs
				.getValue("probability"));
	}

	// <interprophet_result probability="0.00886064"
	// all_ntt_prob="(0,0.0033303,0.00886064)">
	public void startElementinterprophet_result(String nameSpaceUri,
			Attributes attrs) throws MSMSException {
		this.currentPeptideInterProphetProbability = Double.parseDouble(attrs
				.getValue("probability"));
	}

	/*
	 * <search_score name="xcorr" value="0.528"/> <search_score name="deltacn"
	 * value="0.059"/> <search_score name="deltacnstar" value="0.000"/>
	 * <search_score name="spscore" value="29.4"/> <search_score name="sprank"
	 * value="2"/> <search_score name="expect" value="1.00E+00"/>
	 */
	public void startElementsearch_score(String nameSpaceUri, Attributes attrs) {
		String name = attrs.getValue("name");
		String valueStr = attrs.getValue("value");
		if (!valueStr.isEmpty()) {
			if (name.equals("expect")) {
				currentExpectedValue = new Double(valueStr);
				currentPeptide.set_evalue(currentExpectedValue.floatValue());
			}
			// <search_score name="hyperscore" value="232"/>
			if (name.equals("hyperscore")) {
				currentPeptide.set_hypercorr(Float.parseFloat(valueStr));
			}
		}
	}

	// <modification_info modified_peptide="SQRDCR"> <mod_aminoacid_mass
	// position="5" mass="160.030649"/> </modification_info>
	public void startElementmod_aminoacid_mass(String nameSpaceUri,
			Attributes attrs) {
		float massFloat = readMassModification(attrs.getValue("mass"));
		int position = Integer.parseInt(attrs.getValue("position"));
		String aa = currentPeptide.getSequence().substring(position - 1,
				position);
		Float massDiff = massFloat - Utils.getAaMass(aa.charAt(0));

		this.currentPeptide.set_Modifs(aa, position - 1, massDiff);
		// modifs liste for isotopic analysis
		identification.add_modifs_to_liste(new Modifs(massFloat));
	}

	private float readMassModification(String value) {
		// reported Mass modifications are sligthly different depending on the
		// search engine used...
		// take only 3 digits of precision to merge data more easily
		return (float) (Math.round(Double.parseDouble(value) * 1000.0) / 1000.0);
	}

	public Identification get_identifications() {
		return (identification);
	}

	public MzxmlFile getMzxmlFile() {
		return mzXmlFile;
	}

	public FastaFile getFastaFile() {
		return fastaFile;
	}

}
